import datetime as dt
import pandas as pd

# local library that generates the html strings to download NEST data
from nmdb import nest


def main():
    """read data for a single station with NEST into a dataframe"""
    print("NMDB: NEST download tool for a single station")

    start = dt.datetime(2022, 2, 1, 0, 0, 0)
    end = dt.datetime(2022, 2, 28, 23, 59, 59)
    table = "revori"  # virtual table with merge original and revised data

    station = "oulu"  # station short name as used in NMDB
    data = ["p", "u", "c", "e"]  # download pressure, uncorrected, corrected (for pressure) and efficiency corrected data

    download = nest.single(station, table, data, start, end)
    # the download string for the data we selected
    print(download)

    names = data.copy()  # keep the original columns, work only with a copy
    names.insert(0, "start_date_time")
    print(names)
    df = pd.read_table(download, sep=";", comment="#", header=0, names=names)
    print(df)

    # Select only data with very high pressure
    print(df[df['p'] > 1025])

    # Select only data with very low pressure
    print(df[df['p'] < 965])

    df.plot(y=["p"])
    df.plot(y=["u", "e"])

    # Row numbers as x-axis looks ugly, lets use the date instead
    df.index = pd.to_datetime(df['start_date_time'])
    df.plot(y=["u", "c"])


if __name__ == '__main__':
    print("To use this file as starting point for your own script, copy __init.py__ to a new place and edit it to your liking:")
    print("cp ~/.local/lib/python3.9/site-packages/nmdb_nest_single/__init__.py /tmp/single.py")
    print("\n")

    main()



import pandas as pd


def main():
    """read NMDB realtime data into a pandas dataframe"""
    print("NMDB: realtime data")

    realtime = "http://rt.nmdb.eu/realtime.txt"  # NMDB realtime data is available here    
    columns = ['timestamp', 'station', 'countrate']  # list with column names
    df = pd.read_table(realtime, comment="#", sep=";", header=0, names=columns)

    df['timestamp'] = pd.to_datetime(df['timestamp'])  # convert str to datetime
    df = df.astype({'station':'string'})
    df['countrate'] = pd.to_numeric(df['countrate'], errors='coerce')  # None -> NaN
    df.dropna(inplace=True)  # drop rows that contain NaN/None values
    print(df)

    # create a pivot table with station names as column name
    data = df.pivot(index='timestamp', columns='station', values='countrate')

    # run this file in spyder to see the plots
    data.plot()  # plot all the data
    data.ATHN.plot()  # plot a single station
    data[['ATHN', 'OULU']].plot()  # plot multiple stations

    # calculate the mean countrate for one or several stations
    print("mean countrate for ATHN: ", data.ATHN.mean())
    print(data[['ATHN', 'OULU']].mean())
    print("mean countrate:\n", data.mean())


if __name__ == '__main__':
    print("To use this file as starting point for your own script, copy __init.py__ to a new place and edit it to your liking:")
    print("cp ~/.local/lib/python3.9/site-packages/nmdb_realtime/__init__.py /tmp/realtime.py")
    print("\n")

    main()



import datetime as dt
import pandas as pd

# local library that generates the html strings to download NEST data
from nmdb import nest


def main():
    """read data for multiple stations with NEST into a dataframe"""
    print("NMDB: NEST download tool for multiple stations")

    start = dt.datetime(2022, 1, 1, 0, 0, 0)
    end = dt.datetime(2022, 1, 2, 23, 59, 59)
    # NEST returns stations sorted alphabetically!
    station = sorted(["oulu", "kiel2", "jung"])
    # only one data type for multiple stations
    data = "e"
    table = "revori"
    download = nest.multi(station, table, data, start, end)
    print(download)
    header = nest.header(download)
    print(header)

    names = station.copy()
    names.insert(0, "start_date_time")
    df = pd.read_table(download, sep=";", comment="#", header=0, names=names)
    print(df)
    df.plot()

    df.index = pd.to_datetime(df['start_date_time'])
    df.plot()

    df[df['start_date_time'] > "2022-01-02"].plot()

    df[(df['start_date_time'] > "2022-01-02") & (df['start_date_time'] < "2022-01-02 12:00:00") ].plot()

    df[(df['start_date_time'] > "2022-01-02") & (df['kiel2'] < 181) ].plot()

    df[(df['start_date_time'] > "2022-01-02") & (df['kiel2'] < 181) ]



if __name__ == '__main__':
    print("To use this file as starting point for your own script, copy __init.py__ to a new place and edit it to your liking:")
    print("cp ~/.local/lib/python3.9/site-packages/nmdb_nest_multi/__init__.py /tmp/multi.py")
    print("\n")

    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
NEST download tools

Copyright (C) 2022-2023 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

# --------------------------------------------------------------------------
tables = {"e": "corr_for_efficiency",
          "c": "corr_for_pressure",
          "u": "uncorrected",
          "p": "pressure_mbar"}

# --------------------------------------------------------------------------
def dates(dt):
    Y = dt.year
    M = dt.month
    D = dt.day
    h = dt.hour
    m = dt.minute

    return (Y, M, D, h, m)


# --------------------------------------------------------------------------
def multi(station, table, data, start, end):
    """
    query NEST for multiple stations, single data type

    Parameters
    ----------
    station : [str]
        list of station names
    table : str
        revori, ori, or 1h data
    data : str
        corrected (c, e), uncorrected (u), or pressure (p)
    start, end : datetime
        start and end of timeinterval for data

    Returns
    -------
    NEST string

    """
    
    value = "https://www.nmdb.eu/nest/draw_graph.php?wget=1"

    # use a copy of the list as the list is modified by using pop
    stations = station.copy()
    while len(stations):
        s = stations.pop(0)
        value += "&stations[]=%s" %(s.upper())
    value += "&output=ascii"
    if table in ["revori", "ori", "1h"]:
        value += "&tabchoice=%s" %(table)
    else:
        raise ValueError  # table not defined
    try:
        value += "&odtype[]=%s" % (tables[data])
    except:
            raise ValueError  # data type not defined
    value += "&date_choice=bydate"
    value += "&start_year=%04i&start_month=%02i&start_day=%02i&start_hour=%02i&start_min=%02i" % (dates(start))
    value += "&end_year=%04i&end_month=%02i&end_day=%02i&end_hour=%02i&end_min=%02i"  % (dates(end))
    value += "&yunits=0"

    return(value)


# --------------------------------------------------------------------------
def single(station, table, data, start, end):
    """
    query NEST for single station, multiple data types

    Parameters
    ----------
    station : str
        station name
    table : str
        revori, ori, or 1h data
    data : [str]
        corrected (c, e), uncorrected (u), or pressure (p)
    start, end : datetime
        start and end of timeinterval for data

    Raises
    ------
    ValueError
        for undefined tables or data types

    Returns
    -------
    NEST string
    """

    value = "https://www.nmdb.eu/nest/draw_graph.php?wget=1"
    value += "&stations[]=%s" %(station.upper())
    value += "&output=ascii"
    if table in ["revori", "ori", "1h"]:
        value += "&tabchoice=%s" % (table)
    else:
        raise ValueError  # table not defined

    # use a copy of the list as the list is modified by using pop
    dtype = data.copy()
    d = dtype.pop(0)
    try:
        value += "&dtype=%s" % (tables[d])
    except:
        raise ValueError  # data type not defined
    while len(dtype):
        d = dtype.pop(0)
        try:
            value += "&odtype[]=%s" % (tables[d])
        except:
            raise ValueError  # extra data type not defined
    value += "&date_choice=bydate"
    value += "&start_year=%04i&start_month=%02i&start_day=%02i&start_hour=%02i&start_min=%02i" % (dates(start))
    value += "&end_year=%04i&end_month=%02i&end_day=%02i&end_hour=%02i&end_min=%02i"  % (dates(end))
    value += "&yunits=0"

    return(value)


# --------------------------------------------------------------------------
def header(download):
    """
    return NEST header with meta data

    Parameters
    ----------
    download : NEST string

    Returns
    -------
    header
    """
    import urllib.request

    data = urllib.request.urlopen(download)
    header  = ""
    for line in data:
        if line[0] == 35: # NEST headers start with '#' == ASCII 35
            header += line[1:].decode() # remove the '#' from the header

    return(header)


# --------------------------------------------------------------------------

